' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip_IoHandler_002.spin                                                                       *
' *                                                                                                     *
' *******************************************************************************************************

OBJ

  router                        : "AiChip_Router_002"

CON

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Device Routing                                                                                  |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'
 
  ROUTE_NONE                    = router#ROUTE_NONE
  ROUTE_RX                      = router#ROUTE_RX
  ROUTE_TX                      = router#ROUTE_TX
  ROUTE_TV                      = router#ROUTE_TV
  ROUTE_LCD                     = router#ROUTE_LCD
  ROUTE_USB                     = router#ROUTE_USB

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     TV Modes                                                                                        |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  TV_MODE_NTSC                  = router#TV_MODE_NTSC
  TV_MODE_PAL                   = router#TV_MODE_PAL

  TV_MODE_LARGE_FONT            = router#TV_MODE_LARGE_FONT
  TV_MODE_SMALL_FONT            = router#TV_MODE_SMALL_FONT
  
  TV_MODE_FAST_UPDATE           = router#TV_MODE_FAST_UPDATE
  TV_MODE_FLICKER_FREE          = router#TV_MODE_FLICKER_FREE

  TV_MODE_COLOR                 = router#TV_MODE_COLOR
  TV_MODE_COLOUR                = router#TV_MODE_COLOUR
  TV_MODE_MONOCHROME            = router#TV_MODE_MONOCHROME
  
  TV_MODE_INTERLACED            = router#TV_MODE_INTERLACED
  TV_MODE_NON_INTERLACED        = router#TV_MODE_NON_INTERLACED

  TV_MODE_COMPOSITE             = router#TV_MODE_COMPOSITE
  TV_MODE_BASEBAND              = router#TV_MODE_BASEBAND
  TV_MODE_BROADCAST             = router#TV_MODE_BROADCAST

  TV_MODE_FONT_0                = router#TV_MODE_FONT_0
  TV_MODE_FONT_1                = router#TV_MODE_FONT_1
  
  TV_MODE_RUNNING               = router#TV_MODE_RUNNING
      
VAR

  byte numberRadix
  byte numberWidth

' *******************************************************************************************************
' *                                                                                                     *
' *     Device Handling Routines                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

PUB Open( setDeviceType, setBasePin, setMode )

  if setBasePin => 0
    router.Open( setDeviceType, setBasePin, setMode )
  else
    router.Close( setDeviceType )

PUB Close( setDeviceType )

  router.Close( setDeviceType )

PUB Route( setDeviceType )

  result := router.Route( setDeviceType )

PUB getMode( setDeviceType )

  result := router.GetMode( setDeviceType )
          
' *******************************************************************************************************
' *                                                                                                     *
' *     Text Handling and Transmit Routines                                                             *
' *                                                                                                     *
' *******************************************************************************************************

PUB TxText( txtPtr )

  repeat strsize( txtPtr )
    TxByte( byte[ txtPtr++ ] )

PUB TxByte( b )

  router.TxByte( b )
  
PUB TxRawByte( b )

  router.TxRawByte( b )

PUB TxCrLf

  router.TxCrLf
    
PUB Cls

  router.Cls
  
PUB Home

  router.Home
  
PUB TxTab

  router.TxTab
  
PUB TxBackSpace

  router.TxBackSpace
  
' *******************************************************************************************************
' *                                                                                                     *
' *     Number Handling Routines                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

PUB TxNumberFormat( n, radix, width )

  numberRadix := radix
  numberWidth := width
  TxNumber(n)

PUB TxNumber( n ) | radix, digits, k

  radix := numberRadix

  ' Number radix = %000nnnnn   +n      No leading type identifier
  '                %10nnnnnn   $80+n   Logic Analyser Bits + stretching
  '                %11mmmmmm   -n      Leading type identifier

  if ( radix & %1100_0000 ) == %1100_0000
    radix := ( ( ! radix ) + 1 ) & $3F
    case radix
      2  : TxByte("%")
      4  : TxByte("%")
           TxByte("%")
      8  : TxByte("&")
           TxByte("o")
      16 : TxByte("$")

  if radix => %1000_000

    TxLogicAnalyserTrace(n,numberWidth,radix & %0011_1111)

  else

    if n == 0
      repeat numberWidth #> 1
        TxByte("0")
    else

      case radix

        2  : TxRadixNumber(n,31,$1,1,32)
        4  : TxRadixNumber(n,30,$3,2,16)
        8  : TxRadixNumber(n,30,$7,3,11)
        16 : TxRadixNumber(n,28,$F,4,8)

        other : k := 1_000_000_000
                digits := 11
                if n < 0
                  TxByte("-")
                  n := -n
                  digits--
                repeat while k > n
                  k /= 10
                  digits--
                repeat while numberWidth => digits++
                  TxByte("0")
                repeat while k <> 0
                  TxDigit( ( n / k ) // 10 )
                  k /= 10

PRI TxRadixNumber(n,shift,mask,decr,digits)

  repeat while ( n >> shift ) == 0
    shift -= decr
    digits--
  repeat while numberWidth > digits++
    TxByte("0")
  repeat while shift => 0
    TxDigit( ( n >> shift ) & mask )
    shift -= decr

PRI TxDigit( n )

  if ( n += "0" ) > "9"
    n +=  constant( 0 - "9" - 1 + "A" )
  TxRawByte( n )

' *******************************************************************************************************
' *                                                                                                     *
' *     Logic Analyser Trace Handling Routines                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PRI TxLogicAnalyserTrace(bits,digitsToShow,stretch) | bitNumber

  if digitsToShow == 0
    digitsToShow := 32
  else
    if digitsToShow => $80 ' -Ve means right most bits of 16-bit data
      digitsToShow := ( ( ! digitsToshow ) + 1 ) & $7F
      repeat 16
        bits := ( bits << 1 ) | ( bits & 1 )

  repeat bitNumber from 0 to 31
    repeat stretch
      if digitsToShow-- > 0
        TxLogicAnalyserBit(bits,bits)
    if digitsToShow-- > 0
      if bitNumber < 31
        TxLogicAnalyserBit(bits,bits<<1)
      else
        TxLogicAnalyserBit(bits,bits)
      bits <<= 1

  repeat while digitsToShow-- > 0
    TxRawByte(" ")

PRI TxLogicAnalyserBit( thisBit, nextBit )

  TxRawByte( router.GetLogicAnalyserBit( thisBit, nextBit ) )

' *******************************************************************************************************
' *                                                                                                     *
' *     Receive Routines                                                                                *
' *                                                                                                     *
' *******************************************************************************************************

PUB RxByte : rxd

  repeat while ( rxd := RxIfAvailable ) < 0

PUB RxFlush

  repeat while RxIfAvailable => 0

PUB RxIfAvailable : rxd

  router.RxIfAvailable
    
' *******************************************************************************************************
' *                                                                                                     *
' *     Attribute Handling for Video Drivers                                                            *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetColourCount

  result := router.GetColourCount

PUB GetColumnCount

  result := router.GetColumnCount

PUB GetRowCount

  result := router.GetRowCount
  
PUB GetCurrentColumn

  result := router.GetCurrentColumn
  
PUB GetCurrentRow

  result := router.GetCurrentRow
    
' *******************************************************************************************************
' *                                                                                                     *
' *     Colour Palette Handling for Video Drivers                                                       *
' *                                                                                                     *
' *******************************************************************************************************

PUB SetPalette( ptrToPalette )

  router.SetPalette( ptrToPalette )
               
PUB SetPaletteColour( colorIdx, fore, back )

  router.SetPaletteColour( colorIdx, fore, back )

PUB SetColour( useColour )

  router.SetColour( useColour )

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip_IoHandler_002.spin                                                                *
' *                                                                                                     *
' *******************************************************************************************************