' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip_Router_002.spin                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

OBJ

  tv                            : "AiChip_TvText_002"
  serial                        : "AiChip_Serial_002"
  lcd                           : "AiChip_LCD_002"
  usb                           : "AiChip_USB_002"
  
CON

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Device Routing                                                                                  |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'
 
  ROUTE_NONE                    = 0
  ROUTE_RX                      = 1
  ROUTE_TX                      = 2
  ROUTE_TV                      = 3
  ROUTE_LCD                     = 4
  ROUTE_USB                     = 5

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     TV Modes                                                                                        |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'
   
  TV_MODE_NTSC                  = tv#TV_MODE_NTSC
  TV_MODE_PAL                   = tv#TV_MODE_PAL

  TV_MODE_LARGE_FONT            = tv#TV_MODE_LARGE_FONT
  TV_MODE_SMALL_FONT            = tv#TV_MODE_SMALL_FONT

  TV_MODE_FAST_UPDATE           = tv#TV_MODE_FAST_UPDATE
  TV_MODE_FLICKER_FREE          = tv#TV_MODE_FLICKER_FREE

  TV_MODE_COLOR                 = tv#TV_MODE_COLOR
  TV_MODE_COLOUR                = tv#TV_MODE_COLOUR
  TV_MODE_MONOCHROME            = tv#TV_MODE_MONOCHROME
  
  TV_MODE_INTERLACED            = tv#TV_MODE_INTERLACED
  TV_MODE_NON_INTERLACED        = tv#TV_MODE_NON_INTERLACED

  TV_MODE_COMPOSITE             = tv#TV_MODE_COMPOSITE
  TV_MODE_BASEBAND              = tv#TV_MODE_BASEBAND
  TV_MODE_BROADCAST             = tv#TV_MODE_BROADCAST

  TV_MODE_FONT_0                = tv#TV_MODE_FONT_0
  TV_MODE_FONT_1                = tv#TV_MODE_FONT_1

  TV_MODE_RUNNING               = tv#TV_MODE_RUNNING
  
VAR

  byte deviceType

' *******************************************************************************************************
' *                                                                                                     *
' *     Device Handling Routines                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

PUB Open( setDeviceType, setBasePin, setMode )

  deviceType := setDeviceType
  case deviceType
    ROUTE_RX                    : result := serial.OpenRx( setBasePin, setMode )
    ROUTE_TX                    : result := serial.OpenTx( setBasePin, setMode )
    ROUTE_TV                    : result := tv.Open( setBasePin, setMode )
    ROUTE_LCD                   : result := lcd.Open( setBasePin, setMode )
    ROUTE_USB                   : result := usb.Open( setBasePin, setMode )

PUB Close( setDeviceType )

  case setDeviceType
    ROUTE_RX                    : result := serial.CloseRx
    ROUTE_TX                    : result := serial.CloseTx
    ROUTE_TV                    : result := tv.Close
    ROUTE_LCD                   : result := lcd.Close
    ROUTE_USB                   : result := usb.Close

PUB Route( setDeviceType )

  if GetMode( setDeviceType )
    deviceType := setDeviceType
  else
    deviceType := ROUTE_NONE

PUB getMode( setDeviceType )

  case setDeviceType
    ROUTE_RX                    : result := serial.GetModeRx
    ROUTE_TX                    : result := serial.GetModeTx
    ROUTE_TV                    : result := tv.GetMode
    ROUTE_LCD                   : result := lcd.GetMode
    ROUTE_USB                   : result := usb.GetMode
            
' *******************************************************************************************************
' *                                                                                                     *
' *     Text Handling Routines                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB TxByte( b )

  case deviceType
    ROUTE_TX                    : serial.TxByte( b )
    ROUTE_TV                    : tv.TxByte( b )
    ROUTE_LCD                   : lcd.TxByte( b )
    ROUTE_USB                   : usb.TxByte( b )

PUB TxRawByte( b )

  case deviceType
    ROUTE_TX                    : serial.TxRawByte( b )
    ROUTE_TV                    : tv.TxRawByte( b )
    ROUTE_LCD                   : lcd.TxRawByte( b )
    ROUTE_USB                   : usb.TxRawByte( b )

PUB TxCrLf

  case deviceType
    ROUTE_TX                    : serial.TxCrLf
    ROUTE_TV                    : tv.TxCrLf
    ROUTE_LCD                   : lcd.TxCrLf
    ROUTE_USB                   : usb.TxCrLf
    
PUB Cls

  case deviceType
    ROUTE_TX                    : serial.Cls
    ROUTE_TV                    : tv.Cls
    ROUTE_LCD                   : lcd.Cls
    ROUTE_USB                   : usb.Cls
      
PUB Home

  case deviceType
    ROUTE_TX                    : serial.Home
    ROUTE_TV                    : tv.Home
    ROUTE_LCD                   : lcd.Home
    ROUTE_USB                   : usb.Home

PUB TxTab

  case deviceType
    ROUTE_TX                    : serial.TxTab
    ROUTE_TV                    : tv.TxTab
    ROUTE_LCD                   : lcd.TxTab
    ROUTE_USB                   : usb.TxTab

PUB TxBackSpace

  case deviceType
    ROUTE_TX                    : serial.TxBackSpace
    ROUTE_TV                    : tv.TxBackSpace
    ROUTE_LCD                   : lcd.TxBackSpace
    ROUTE_USB                   : usb.TxBackSpace

' *******************************************************************************************************
' *                                                                                                     *
' *     Logic Analyser Trace Handling Routines                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetLogicAnalyserBit( thisBit, nextBit )

  case deviceType
    ROUTE_TX                    : result := serial.GetLogicAnalyserBit( thisBit, nextBit )
    ROUTE_TV                    : result := tv.GetLogicAnalyserBit( thisBit, nextBit )
    ROUTE_LCD                   : result := lcd.GetLogicAnalyserBit( thisBit, nextBit )
    ROUTE_USB                   : result := usb.GetLogicAnalyserBit( thisBit, nextBit )

' *******************************************************************************************************
' *                                                                                                     *
' *     Receive Routines                                                                                *
' *                                                                                                     *
' *******************************************************************************************************

PUB RxIfAvailable : rxd

  rxd--
  case deviceType
    ROUTE_RX                    : rxd := serial.RxIfAvailable
    ROUTE_USB                   : rxd := usb.RxIfAvailable
      
' *******************************************************************************************************
' *                                                                                                     *
' *     Attribute Handling for Video Drivers                                                            *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetColourCount

  case deviceType
    ROUTE_TV                    : result := tv.GetColourCount

PUB GetColumnCount

  case deviceType
    ROUTE_TX                    : result := serial.GetColumnCount
    ROUTE_TV                    : result := tv.GetColumnCount
    ROUTE_LCD                   : result := lcd.GetColumnCount
    ROUTE_USB                   : result := usb.GetColumnCount

PUB GetRowCount

  case deviceType
    ROUTE_TX                    : result := serial.GetColumnCount
    ROUTE_TV                    : result := tv.GetRowCount
    ROUTE_LCD                   : result := lcd.GetRowCount
    ROUTE_USB                   : result := usb.GetColumnCount
  
PUB GetCurrentColumn

  case deviceType
    ROUTE_TX                    : result := serial.GetColumnCount
    ROUTE_TV                    : result := tv.GetCurrentColumn
    ROUTE_LCD                   : result := lcd.GetCurrentColumn
    ROUTE_USB                   : result := usb.GetColumnCount
  
PUB GetCurrentRow

  case deviceType
    ROUTE_TX                    : result := serial.GetColumnCount
    ROUTE_TV                    : result := tv.GetCurrentRow
    ROUTE_LCD                   : result := lcd.GetCurrentRow
    ROUTE_USB                   : result := usb.GetColumnCount
    
' *******************************************************************************************************
' *                                                                                                     *
' *     Colour Palette Handling for Video Drivers                                                       *
' *                                                                                                     *
' *******************************************************************************************************

PUB SetPalette( ptrToPalette )

  case deviceType
    ROUTE_TV                    : tv.SetPalette( ptrToPalette )
               
PUB SetPaletteColour( colorIdx, fore, back )

  case deviceType
    ROUTE_TV                    : tv.SetPaletteColour( colorIdx, fore, back )

PUB SetColour( useColour )

  case deviceType
    ROUTE_TV                    : tv.SetColour( useColour )

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip_Router_002.spin                                                                   *
' *                                                                                                     *
' *******************************************************************************************************