' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip_TvText_002.spin                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

OBJ

  tv_LargeFont                  : "AiChip_LargeFont_001"
  tv_SmallFont                  : "AiChip_SmallFont_002"
  
CON

  TV_MODE_NTSC                  = %0000_0000
  TV_MODE_PAL                   = %0000_0001

  TV_MODE_LARGE_FONT            = %0000_0000
  TV_MODE_SMALL_FONT            = %0000_0010

  TV_MODE_FAST_UPDATE           = %0000_0000
  TV_MODE_FLICKER_FREE          = %0000_0100

  TV_MODE_COLOR                 = %0000_0000
  TV_MODE_COLOUR                = %0000_0000
  TV_MODE_MONOCHROME            = %0000_1000
  
  TV_MODE_INTERLACED            = %0000_0000
  TV_MODE_NON_INTERLACED        = %0001_0000

  TV_MODE_COMPOSITE             = %0000_0000
  TV_MODE_BASEBAND              = %0000_0000
  TV_MODE_BROADCAST             = %0010_0000

  TV_MODE_FONT_0                = %0000_0000
  TV_MODE_FONT_1                = %0100_0000

  TV_MODE_RUNNING               = %1000_0000
    
  RGB_BLACK                     = $02 
  RGB_GREY                      = $03
  RGB_WHITE                     = $06
  RGB_BRIGHT_WHITE              = $07
  RGB_CYAN                      = $3D
  RGB_LIGHT_CYAN                = $3E 
  RGB_LIGHT_GREEN               = $5E 
  RGB_LIGHT_YELLOW              = $8E 
  RGB_LIGHT_RED                 = $CE
  RGB_LIGHT_MAGENTA             = $FE 

  RGB_RED                       = $BC          
  RGB_YELLOW                    = $8D
  RGB_GREEN                     = $5C
  RGB_BLUE                      = $1C
  RGB_MAGENTA                   = $FC
  RGB_LIGHT_BLUE                = $1E

  CR                            = $0D
  LF                            = $0A
  TAB                           = $09
  BKS                           = $08
  
VAR

  byte rows
  byte columns

  byte row
  byte column
  byte colour

  byte mode
  
  byte heldChar
  byte escapeChar
  byte boxHeight
  word number

' *******************************************************************************************************
' *                                                                                                     *
' *     Device Handling Routines                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

PUB Open( setBasePin, setMode )

  setMode |= TV_MODE_RUNNING

  if mode <> setMode

    Close
    
    if setMode & TV_MODE_SMALL_FONT
      mode := tv_SmallFont.Open( setBasePin, setMode, @PALETTE )
      rows := tv_SmallFont.GetRowCount
      columns := tv_SmallFont.GetColumnCount
    else
      mode := tv_LargeFont.Open( setBasePin, setMode, @PALETTE )
      rows := tv_LargeFont.GetRowCount
      columns := tv_LargeFont.GetColumnCount

    escapeChar := 0
    heldChar := 0

  return mode
  
PUB Close

  if mode
    if mode & TV_MODE_SMALL_FONT
      tv_SmallFont.Close
    else
      tv_LargeFont.Close
    mode := 0

PUB GetMode

  result := mode
           
' *******************************************************************************************************
' *                                                                                                     *
' *     Text Handling Routines                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB TxByte( b ) | nextHeldChar

  ' "^^"                        Print "^"
  ' "^A", row, col              AT           1..13, 1..40
  ' "^B", high, wide            BOX          2..13, 2..40  use high=0 for max height
  ' "^C", col                   COLUMN       1..40
  ' "^E"                        ERASE LINE
  ' "^F", color                 FORE COLOR   0..15
  ' "^H"                        HOME
  ' "^I"                        TAB
  ' "^L"                        LINEFEED
  ' "^N"                        NEW                        clears screen
  ' "^P", time                  PAUSE        0..65535 mS   
  ' "^R", col                   ROW          1..13         also sets column 1
  ' "^X", across                X-AXIS       0..39
  ' "^Y", up                    Y-AXIS       12..0
  ' "^*", char                  Print char   0..255
  
  ' "^#numberF"                 Equivalent of "^F",number ( 'F' can be any valid letter of the above )

  if escapeChar == 0
    case b
      "^", "\", "~" : escapeChar := b

  nextheldChar := 0
  case heldChar

    0 :
      if b == escapeChar
        nextHeldChar := escapeChar
      else
        case b
          CR    : column := 0
          LF    : TxCrLf
          TAB   : TxTab
          BKS   : TxBackSpace
          other :
            if b == "0"
              b := "O"
            TxRawByte(b)

    escapeChar :
      case b
        escapeChar : TxRawByte(b)
        "E"        : heldChar := column
                     repeat while column < columns
                       TxRawByte(" ")
                     column := heldChar
        "H"        : Home
        "I"        : TxTab
        "L"        : TxCrLf
        "N"        : Cls
        "#"        : number := 0
                     nextHeldChar := "#"
        other      : nextHeldChar := Ucase(b)

    "A"   : row := (b-1) // rows
            nextHeldChar := "C"
    "B"   : boxHeight := b
            nextHeldChar := "b"
    "b"   : if boxHeight == 0
              boxHeight := rows
            if mode & TV_MODE_SMALL_FONT
              BoxLine(b,tv_SmallFont#CHR_TOP_LEFT,tv_SmallFont#CHR_TOP_MID,tv_SmallFont#CHR_TOP_RIGHT)
              repeat while boxHeight-- > 2
                if b == 0
                  TxRawByte(tv_SmallFont#CHR_LFT_BAR)
                  column := columns-1
                  TxRawByte(tv_SmallFont#CHR_LFT_BAR)
                else
                  BoxLine(b,tv_SmallFont#CHR_LFT_BAR," ",tv_SmallFont#CHR_LFT_BAR)
              BoxLine(b,tv_SmallFont#CHR_BOT_LEFT,tv_SmallFont#CHR_TOP_MID,tv_SmallFont#CHR_BOT_RIGHT)
            else
              BoxLine(b,tv_LargeFont#CHR_TOP_LEFT,tv_LargeFont#CHR_TOP_MID,tv_LargeFont#CHR_TOP_RIGHT)
              repeat while boxHeight-- > 2
                if b == 0
                  TxRawByte(tv_LargeFont#CHR_LFT_BAR)
                  column := columns-1
                  TxRawByte(tv_LargeFont#CHR_LFT_BAR)
                else
                  BoxLine(b,tv_LargeFont#CHR_LFT_BAR," ",tv_LargeFont#CHR_LFT_BAR)
              BoxLine(b,tv_LargeFont#CHR_BOT_LEFT,tv_LargeFont#CHR_TOP_MID,tv_LargeFont#CHR_BOT_RIGHT)
    "C"   : column := (b-1) // columns
    "F"   : SetColour( b )
    "P"   : PauseMs(b)
    "R"   : row := (b-1) // rows
            column := 0
    "X"   : column := b // columns
    "Y"   : row := rows - ( b // rows )

    "#"   : if ( b < "0" ) or ( b > "9" )
              if b == ","
                boxHeight := number
                number := 0
                nextHeldChar := "#"
              else
                heldChar := b
                TxByte(number)
                nextHeldChar := 0
            else
              number := ( number * 10 ) + ( b-"0")
              nextHeldChar := "#"

    other : TxRawByte(b)

  heldChar := nextHeldChar

PUB TxRawByte( b )

  if column => columns
    TxCrLf
  if row => rows
    Scroll
  PokeChar( row, column, colour, b )
  column++

PUB TxCrLf

  column := 0
  if row++ > rows
    Scroll
    
PUB Cls

  if mode & TV_MODE_SMALL_FONT
    tv_SmallFont.Cls
  else
    tv_LargeFont.Cls
  colour := 0
  Home

PUB Home

  column := row := 0

PUB TxTab

  repeat
    TxRawByte(" ")
  while ( column // 9 ) <> 0

PUB TxBackSpace

  if column
    column--
  else
    column := 0
    if row
      row--
  PokeChar(row,column,colour," ")

PRI BoxLine( width, lhs, mid, rhs ) | tmp
  if width == 0
    width := columns
  tmp := width
  TxRawByte(lhs)
  repeat while tmp-- > 2
    TxRawByte(mid)
  TxRawByte(rhs)
  if mode & TV_MODE_SMALL_FONT
    if lhs <> tv_SmallFont#CHR_BOT_LEFT
      row++
      column -= width
  else
    if lhs <> tv_LargeFont#CHR_BOT_LEFT
      row++
      column -= width

PRI Scroll

  repeat while row => rows
    if mode & TV_MODE_SMALL_FONT
      tv_SmallFont.Scroll
    else
      tv_LargeFont.Scroll
    row--

PRI PokeChar(setRow,setCol,useColour,c)

  if mode & TV_MODE_SMALL_FONT
    tv_SmallFont.PokeChar(setRow,setCol,useColour,c)
  else
    tv_LargeFont.PokeChar(setRow,setCol,useColour,c)

PRI Ucase( ch )

  return ( ch & $5F ) | $40
                                
' *******************************************************************************************************
' *                                                                                                     *
' *     Logic Analyser Trace Handling Routines                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetLogicAnalyserBit(thisBit,nextBit)

  if mode & TV_MODE_SMALL_FONT
    result := tv_SmallFont.GetLogicAnalyserBit( thisBit, nextBit )
  else
    result := tv_LargeFont.GetLogicAnalyserBit( thisBit, nextBit )

' *******************************************************************************************************
' *                                                                                                     *
' *     Attribute Handling for Video Drivers                                                            *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetColourCount

  if mode & TV_MODE_SMALL_FONT
    result := tv_SmallFont.GetColourCount
  else
    result := tv_LargeFont.GetColourCount

PUB GetColumnCount

  result := columns
    
PUB GetRowCount

  result := rows
  
PUB GetCurrentColumn

  result := column

PUB GetCurrentRow

  result := row

' *******************************************************************************************************
' *                                                                                                     *
' *     Colour Palette Handling for Video Drivers                                                       *
' *                                                                                                     *
' *******************************************************************************************************

PUB SetPalette( ptrToPalette )

  if mode & TV_MODE_SMALL_FONT
    tv_SmallFont.SetPalette( ptrToPalette )
  else
    tv_LargeFont.SetPalette( ptrToPalette )
              
PUB SetPaletteColour( colorIdx, fore, back )

  if mode & TV_MODE_SMALL_FONT
    tv_SmallFont.SetPaletteColour( colorIdx, fore, back )
  else
    tv_LargeFont.SetPaletteColour( colorIdx, fore, back )

PUB SetColour( useColour )

  colour := useColour

' *******************************************************************************************************
' *                                                                                                     *
' *     Timing Routines                                                                                 *
' *                                                                                                     *
' *******************************************************************************************************

PRI PauseMs( mS )

  waitcnt( ( clkfreq / 1000 ) * mS + cnt )

DAT

' *******************************************************************************************************
' *                                                                                                     *
' *     Colour Palette for Video Drivers                                                                *
' *                                                                                                     *
' *******************************************************************************************************

' Basic QB-style colours
'
'       0 White                 8  Grey
'       1 Blue                  9  Light Blue
'       2 Green                 10 Light Green
'       3 Cyan                  11 Light Cyan
'       4 Red                   12 Light Red
'       5 Magenta               13 Light Magenta
'       6 Yellow                14 Light Yellow
'       7 Bright White          15 White on Red
        
PALETTE byte    RGB_BLACK              ' Background

        byte    RGB_WHITE              ' 0
        byte    RGB_BLUE               ' 1
        byte    RGB_GREEN              ' 2  
        byte    RGB_CYAN               ' 3  
        byte    RGB_RED                ' 4  
        byte    RGB_MAGENTA            ' 5  
        byte    RGB_YELLOW             ' 6  
        byte    RGB_BRIGHT_WHITE       ' 7
          
        byte    RGB_GREY               ' 8 
        byte    RGB_LIGHT_BLUE         ' 9
        byte    RGB_LIGHT_GREEN        ' 10
        byte    RGB_LIGHT_CYAN         ' 11
        byte    RGB_LIGHT_RED          ' 12
        byte    RGB_LIGHT_MAGENTA      ' 13
        byte    RGB_LIGHT_YELLOW       ' 14

        byte    RGB_RED                ' 15 - Background with foreground colour 0

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip_TvText_002.spin                                                                   *
' *                                                                                                     *
' *******************************************************************************************************